/*
Copyright (C) <2008>  Guido Steiner and Detlef Wolf
contact: fam@wolfol.de
This is free software -- GNU LGPL license applies.
see http://www.gnu.org/licenses/lgpl.txt for details
*/

import java.lang.Math ;

public class PendulumModel {

  // anchor
  public double a=0.08 ;
  public double b=0.08 ;
  double ma=0.3 ;
  double mb=0.3 ;
  
  // 3 pendulums
  double m1 = 0.2 ;
  double m2 = 0.2 ;
  double m3 = 0.2 ;
  public double l1 = 0.05 ;
  public double l2 = 0.05 ;
  public double l3 = 0.05 ;

  // start conditions
  public double t0=160 *Math.PI/180 ;
  double dt0=0;
  double ddt0=0;
  public double t1=20 *Math.PI/180;
  double dt1=0;
  double ddt1=0;
  public double t2=0;
  double dt2=0;
  double ddt2=0;
  public double t3=-50 *Math.PI/180;
  double dt3=0;
  double ddt3=0;

  // time step in seconds
  public double t=0.0000005 ;

  // constants
  double g=9.81;
  double A=-0.5*mb*g*b ;
  double L=2/3*ma*a*a + 1/3*mb*b*b ;
  double f1=1/ ( L+(m1+m3)*a*a+m2*b*b ) ;

  // derived values: energy
  public double ekin ;
  public double epot ;
  public double etot ;

  public PendulumModel() {
    ;
  }

  public void setStepTime(double t) {
    this.t = t ;
  }
  
  public double getStepTime() {
    return t ;
  }

  public void step() {
    // compute one step
    ddt0 = f1 * (     m1*a*l1*Math.sin(t1-t0)*ddt1 
                    - m2*b*l2*Math.cos(t0-t2)*ddt2 
                    - m3*a*l3*Math.sin(t3-t0)*ddt3 
                    + m1*a*l1*Math.cos(t0-t1)*dt1*dt1 
                    - m2*b*l2*Math.sin(t0-t2)*dt2*dt2 
                    - m3*a*l3*Math.cos(t3-t0)*dt3*dt3 
                    + (A-m2*g*b)*Math.sin(t0) 
                    + (m1-m3)*g*a*Math.cos(t0) ) ;
    
    ddt1 = -1/l1 * (a*Math.sin(t0-t1)*ddt0 + a*Math.cos(t0-t1)*dt0*dt0 + g*Math.sin(t1) ) ;
    
    ddt2 = -1/l2 * (b*Math.cos(t0-t2)*ddt0 - b*Math.sin(t0-t2)*dt0*dt0 + g*Math.sin(t2) ) ;
    
    ddt3 = -1/l3 * (a*Math.sin(t3-t0)*ddt0 - a*Math.cos(t3-t0)*dt0*dt0 + g*Math.sin(t3) ) ;
    
    dt0 = dt0 + ddt0*t ;
    t0 = t0 + dt0*t + 0.5*ddt0*t*t ;
    dt1 = dt1 + ddt1*t ;
    t1 = t1 + dt1*t + 0.5*ddt1*t*t ;
    dt2 = dt2 + ddt2*t ;
    t2 = t2 + dt2*t + 0.5*ddt2*t*t ;
    dt3 = dt3 + ddt3*t ;
    t3 = t3 + dt3*t + 0.5*ddt3*t*t ;
    
    ekin= 0.5*L*dt0*dt0 
      + (m1/2)*(a*a*dt0*dt0+l1*l1*dt1*dt1+2*a*l1*dt0*dt1*Math.sin(t0-t1)) 
      + (m2/2)*(b*b*dt0*dt0+l2*l2*dt2*dt2+2*b*l2*dt0*dt2*Math.cos(t0-t2)) 
      + (m3/2)*(a*a*dt0*dt0+l3*l3*dt3*dt3+2*a*l3*dt0*dt3*Math.sin(t3-t0)) ;
    
    epot= A*Math.cos(t0) 
      - m1*g*(a*Math.sin(t0)+l1*Math.cos(t1)) 
      - m2*g*(b*Math.cos(t0)+l2*Math.cos(t2)) 
      + m3*g*(a*Math.sin(t0)-l3*Math.cos(t3)) ;
    
    etot = epot + ekin ;
    
    /*
      System.out.print("x0=" + x0 + " y0=" + y0) ;
      System.out.print("  x1=" + x1 + " y1=" + y1) ;
      System.out.println("  x2=" + x2 + " y2=" + y2) ;
    */
  } 

  public boolean reveal = false ;

  public void toggle() {
    reveal = !reveal ;
  }
}




